import h5py
import torch
import numpy as np
from torch.utils.data import Dataset, DataLoader
from transformers import LongformerTokenizer, LongformerForSequenceClassification, Trainer, TrainingArguments
from sklearn.metrics import precision_recall_fscore_support, accuracy_score, confusion_matrix

def compute_metrics(p):
    preds, labels = p
    preds = np.argmax(preds, axis=1)
    acc = accuracy_score(labels, preds)
    precision, recall, f1, _ = precision_recall_fscore_support(labels, preds, average='binary', zero_division=0)
    #cm = confusion_matrix(labels, preds)
    return {
        'accuracy': acc,
        'f1': f1,
        'precision': precision,
        'recall': recall
        #'cm' : cm
    }

class RobustDataset(Dataset):
    def __init__(self, filepath):
        self.h5 = h5py.File(filepath, 'r')
        self.keys = list(self.h5.keys())
        
    def __getitem__(self, idx):
        tensors = {}
        for key in self.keys:
            key_name = key
            if key == 'global_attention_masks':
                key_name = key[:-1]
            tensors[key_name] = torch.tensor(self.h5[key][idx]).to(torch.int64)
        return tensors
            
    def __len__(self):
        return len(self.h5['labels'])



test_ds = RobustDataset('datasets/longformer/2048msmarco/test_encodings.hdf5')
train_ds = RobustDataset('datasets/longformer/2048msmarco/train_encodings.hdf5')
model =LongformerForSequenceClassification.from_pretrained('allenai/longformer-base-4096')


training_args = TrainingArguments(
    output_dir='./results/longformer/2048msmarco',          # output directory
    overwrite_output_dir=True,
    num_train_epochs=5,              # total number of training epochs
    per_device_train_batch_size=2,  # batch size per device during training
    per_device_eval_batch_size=2,   # batch size for evaluation
    warmup_steps=5000,                # number of warmup steps for learning rate scheduler
    weight_decay=0.1,               # strength of weight decay
    logging_dir='./logs',            # directory for storing logs
    logging_steps=1000,
    learning_rate=3e-6,
    save_strategy="epoch",
    logging_first_step=True,
    evaluation_strategy="epoch",
    gradient_accumulation_steps=4
)

trainer = Trainer(
    model=model,                         # the instantiated 🤗 Transformers model to be trained
    args=training_args,                  # training arguments, defined above
    train_dataset=train_ds,         # training dataset
    eval_dataset=test_ds,             # evaluation dataset
    compute_metrics=compute_metrics
)

trainer.train()
